@@ -32,8 +32,12 @@ Archive::SevenZip - Read/write 7z , zip , ISO9960 and other archives
 
 =cut
 
-use vars qw(%sevenzip_charsetname %class_defaults $VERSION @EXPORT_OK %EXPORT_TAGS);
-$VERSION= '0.06';
+use vars qw(
+    %sevenzip_charsetname
+    %sevenzip_stdin_support
+    %class_defaults
+    $VERSION @EXPORT_OK %EXPORT_TAGS);
+$VERSION= '0.07';
 
 # Archive::Zip API
 # Error codes
@@ -65,6 +69,15 @@ use constant COMPRESSION_DEFLATED      => 'Deflate';   # file is Deflated
     '' => 'DOS', # dunno what the appropriate name would be
 );
 
+%sevenzip_stdin_support = (
+    #'7z'   => 1,
+    'xz'    => 1,
+    'lzma'  => 1,
+    'tar'   => 1,
+    'gzip'  => 1,
+    'bzip2' => 1,
+);
+
 if( $^O !~ /MSWin/ ) {
     # Wipe all filesystem encodings because my Debian 7z 9.20 doesn't understand them
     $sevenzip_charsetname{ $_ } = ''
@@ -229,7 +242,10 @@ sub list {
     };
     my $cmd = $self->get_command( command => "l", options => ["-slt"], %options );
     
-    my $fh = $self->run($cmd, encoding => $options{ fs_encoding } );
+    my $fh = $self->run($cmd,
+        encoding => $options{ fs_encoding },
+        stdin_fh => $options{ fh },
+     );
     my @output = <$fh>;
     my %results = (
         header => [],
@@ -312,6 +328,7 @@ sub openMemberFH {
 sub content {
     my( $self, %options ) = @_;
     my $fh = $self->openMemberFH( %options );
+    binmode $fh;
     local $/;
     <$fh>
 }
@@ -424,6 +441,7 @@ sub get_command {
         $options{ command },
         @charset,
         add_quotes( @{ $options{ options }} ),
+        "--",
         add_quotes( $options{ archivename } ),
         add_quotes( @{ $options{ members }} ),
     ];
@@ -433,7 +451,7 @@ sub run {
     my( $self, $cmd, %options )= @_;
     
     my $mode = '-|';
-    if( defined $options{ stdin }) {
+    if( defined $options{ stdin } || defined $options{ stdin_fh }) {
         $mode = '|-';
     };
 
@@ -448,8 +466,10 @@ sub run {
         CORE::open( my $fh_err, '>', File::Spec->devnull )
             or warn "Couldn't redirect child STDERR";
         my $errh = fileno $fh_err;
+        my $fh_in = $options{ stdin_fh };
         # We accumulate zombie PIDs here, ah well.
-        my $pid = open3( my $fh_in, my $fh_out, '>&' . $errh, @$cmd)
+        $SIG{'CHLD'} = 'IGNORE';
+        my $pid = open3( $fh_in, my $fh_out, '>&' . $errh, @$cmd)
             or croak "Couldn't launch [$mode @$cmd]: $!/$?";
         if( $mode eq '|-' ) {
             $fh = $fh_in;
@@ -466,6 +486,10 @@ sub run {
     if( $options{ stdin }) {
         print {$fh} $options{ stdin };
         close $fh;
+
+    } elsif( $options{ stdin_fh } ) {
+        close $fh;
+
     } elsif( $options{ skip }) {
         for( 1..$options{ skip }) {
             # Read that many lines
@@ -493,6 +517,7 @@ sub wait {
     while( <$fh>) {
         warn $_ if ($options{ verbose } || $self->{verbose})
     };
+    wait; # reap that child
 }
 
 =head2 C<< $ar->add_scalar >>
@@ -501,9 +526,10 @@ sub wait {
 
 Adds a scalar as an archive member.
 
-Unfortunately, 7zip doesn't reliably read archive members from STDIN,
-so the scalar will be written to a tempfile, added to the archive and then
-renamed in the archive.
+Unfortunately, 7zip only reads archive members from STDIN
+for  xz, lzma, tar, gzip and bzip2 archives.
+In the other cases, the scalar will be written to a tempfile, added to the
+archive and then renamed in the archive.
 
 This requires 7zip version 9.30+
 
@@ -512,42 +538,50 @@ This requires 7zip version 9.30+
 sub add_scalar {
     my( $self, $name, $scalar )= @_;
     
-    # 7zip doesn't really support reading archive members from STDIN :-(
-    my($fh, $tempname) = tempfile;
-    binmode $fh, ':raw';
-    print {$fh} $scalar;
-    close $fh;
+    if( $sevenzip_stdin_support{ $self->{type} } ) {
+        my $cmd = $self->get_command(
+            command => 'a',
+            archivename => $self->archive_or_temp,
+            members => ["-si$name"],
+        );
+        my $fh = $self->run( $cmd,
+            binmode => ':raw',
+            stdin   => $scalar,
+            verbose => 1,
+        );
+
+    } else {
     
-    # Only supports 7z archive type?!
-    # 7zip will magically append .7z to the filename :-(
-    my $cmd = $self->get_command(
-        command => 'a',
-        archivename => $self->archive_or_temp,
-        members => [$tempname],
-        #options =>  ],
-    );
-    $fh = $self->run( $cmd );
-    $self->wait($fh);
-    
-    unlink $tempname
-        or warn "Couldn't unlink '$tempname': $!";
-    
-    # Hopefully your version of 7zip can rename members (9.30+):
-    $cmd = $self->get_command(
-        command => 'rn',
-        archivename => $self->archive_or_temp,
-        members => [basename($tempname), $name],
-        #options =>  ],
-    );
-    $fh = $self->run( $cmd );
-    $self->wait($fh);
-    
-    # Once 7zip supports reading from stdin, this will work again:
-    #my $fh = $self->run( $cmd,
-    #    binmode => ':raw',
-    #    stdin => $scalar,
-    #    verbose => 1,
-    #);
+        # 7zip doesn't really support reading archive members from STDIN :-(
+        my($fh, $tempname) = tempfile;
+        binmode $fh, ':raw';
+        print {$fh} $scalar;
+        close $fh;
+        
+        # Only supports 7z archive type?!
+        # 7zip will magically append .7z to the filename :-(
+        my $cmd = $self->get_command(
+            command => 'a',
+            archivename => $self->archive_or_temp,
+            members => [$tempname],
+            #options =>  ],
+        );
+        $fh = $self->run( $cmd );
+        $self->wait($fh);
+        
+        unlink $tempname
+            or warn "Couldn't unlink '$tempname': $!";
+        
+        # Hopefully your version of 7zip can rename members (9.30+):
+        $cmd = $self->get_command(
+            command => 'rn',
+            archivename => $self->archive_or_temp,
+            members => [basename($tempname), $name],
+            #options =>  ],
+        );
+        $fh = $self->run( $cmd );
+        $self->wait($fh);
+    };
 };
 
 =head2 C<< $ar->add_directory >>
@@ -661,12 +695,48 @@ sub add {
     };
 };
 
+=head2 C<< ->archiveZipApi >>
+
+  my $ar = Archive::SevenZip->archiveZipApi(
+      find => 1,
+      archivename => $archivename,
+      verbose => $verbose,
+  );
+  print "$_\n" for $ar->list_files;
+
+This is an alternative constructor that gives you an API
+that is somewhat compatible with the API of L<Archive::Zip>.
+See also L<Archive::SevenZip::API::ArchiveZip>.
+
+=cut
+
 sub archiveZipApi {
     my( $class, %options ) = @_;
     require Archive::SevenZip::API::ArchiveZip;
     Archive::SevenZip::API::ArchiveZip->new( %options )
 }
 
+=head2 C<< ->archiveTarApi >>
+
+  my $ar = Archive::SevenZip->archiveTarApi(
+      find => 1,
+      archivename => $archivename,
+      verbose => $verbose,
+  );
+  print "$_\n" for $ar->list_files;
+
+This is an alternative constructor that gives you an API
+that is somewhat compatible with the API of L<Archive::Tar>.
+See also L<Archive::SevenZip::API::ArchiveTar>.
+
+=cut
+
+sub archiveTarApi {
+    my( $class, %options ) = @_;
+    require Archive::SevenZip::API::ArchiveTar;
+    Archive::SevenZip::API::ArchiveTar->new( %options )
+}
+
 package Path::Class::Archive::Handle;
 use strict;
 
@@ -695,6 +765,10 @@ L<File::Unpack> - also supports unpacking from 7z archives
 
 L<Compress::unLZMA> - uncompressor for the LZMA compression method used by 7z
 
+L<Archive::Libarchive::Any>
+
+L<Archive::Any>
+
 =head1 REPOSITORY
 
 The public repository of this module is 
